/* Copyright 1996, Jan D. Wolter and Steven R. Weiss, All Rights Reserved. */

/* Read conference Configuration files.
 *
 * Mar 15, 1993 - Jan Wolter:  Original version
 * May 29, 1993 - Jan Wolter:  Revised to read conference modes.
 * Dec  2, 1995 - Jan Wolter:  Ansification
 * Jan  2, 1996 - Jan Wolter:  BackTalk version
 */

#include "backtalk.h"
#include <ctype.h>

#include "ps_config.h"
#include "ps_conflist.h"
#include "str.h"
#include "vstr.h"

#ifdef DMALLOC
#include "dmalloc.h"
#endif

#define CONFIG		"/config"

/* Text modes - These are a Yapp-ism.  We support them for Yapp compatibility.
 * Mode can have on primary keyword and any number of non-primary keywords.
 * Numeric mode is just the sum of the text modes in this table.
 */

struct textmodes {
    char *name;		    /* keyword */
    int prime;		    /* is it a primary keyword? */
    int value;		    /* value */
} textmode[]= {
    {"public",     1,   0},    /* Public conference */
    {"ulist",      1,   4},    /* Ulisted conference */
    {"password",   1,   5},    /* Password protected conference */
    {"paranoid",   1,   6},    /* Password and ulist conference */
    {"protected",  1,   8},    /* Public, but item files depermitted */
    {"readonly",   0,  16},    /* Fishbowl */
    {"fishbowl",   0,  16},    /* Fishbowl */
    {"maillist",   0,  64},    /* mailing list - not supported */
    {"registered", 0, 128},    /* registered mailing list - not supported */
    {"noenter",    0, 256},    /* only fw may enter new items */
    {"acl",        1,  -1},    /* Force getting mode from acl file */
    {"access",     1,  -2},    /* Force getting mode from access file */
    {NULL,         0,   0} };


/* PARSE_TEXTMODE -- Parse a line of text modes.  The passed in pointer should
 * point to the first non-blank character of the line.  Returns NULL on
 * success, or a pointer to an error message in dynamic memory on failure.
 */

char *parse_textmode(char *b, int *mode)
{
    int i, primemode= -1;
    char *e;
    char olde;

    *mode= 0;
    while (*b != '\0')
    {
	e= firstin(b," \t\n\r");
	olde= *e;
	*e= '\0';
	for (i= 0; textmode[i].name != NULL; i++)
	    if (!strcmp(b,textmode[i].name))
	    {
		if (textmode[i].prime)
		{
		    if (primemode == -1)
			primemode= textmode[i].value;
		    else if ((primemode == 4 && textmode[i].value == 5) ||
			     (primemode == 5 && textmode[i].value == 4))
			/* We tolerate 'ulist password' for 'paranoid' */
			primemode= 6;
		    else
			return strdup("Illegal conference mode - "
			    "multiple primary keywords\n");
		    *mode= primemode;
		}
		else
		    *mode+= textmode[i].value;
		break;
	    }

	if (textmode[i].name == NULL)
	{
	    char *err;
	    asprintf(&err,"Illegal conference mode - bad keyword %s\n",b);
	    return err;
	}

	*e= olde;
	b= firstout(e+1," \t\n\r");
    }
    return NULL;
}


/* READCONFIG -- Given the directory path for a conference, return the name
 * of the participation file, the list of fairwitnesses, the  mode of the
 * conference and the conference title.  Modes 4, 5 and 6 indicate closed
 * conferences of various kinds.  Pfname and fw point to storage areas of at
 * least BFSZ bytes.  Title set to null if no conference title is set or is
 * set to point to a conference title in static storage.  If pfname, fw, mode,
 * or title are passed in as NULL, they aren't set.
 *
 * Missing fairwitness, mode and title lines can be tolerated.
 */


void readconfig(char *dir, char *pfname, char *fw,
		int *mode, char *raw_mode, char **title)
{
    FILE *fp;
    static char buf[BFSZ];
    char *b;

    /* Set some defaults */
    if (fw != NULL) fw[0]= '\0';
    if (mode != NULL) *mode= 0;
    if (title != NULL) *title= NULL;

    strcpy(buf,dir);
    strcat(buf,CONFIG);

    if ((fp= fopen(buf,"r")) == NULL)
	die("could not open %s",buf);

    /* Read and check magic code number */
    if (fgets(buf,BFSZ,fp) == NULL || strcmp(buf,"!<pc02>\n"))
	die("Bad config file version number: %s", buf);

    /* Read participation file name */
    if (fgets(buf,BFSZ,fp) == NULL)
	die("conference configuration file is missing name of "
	    "participation file\n");

    if ((b= strchr(buf,'\n')) == NULL)
	die("conference configuration file has overly long "
	    "participation file name\n");
    *b = '\0';
    if (pfname != NULL) strcpy(pfname,buf);

    /* Read the time limit line - this is always zero, so who cares? */
    fgets(buf,BFSZ,fp);

    /* Read the list of fairwitnesses */
    if (fgets(buf,BFSZ,fp) != NULL && fw != NULL)
    {
	if ((b= strchr(buf,'\n')) == NULL)
	    die("conference configuration file has overly long "
		"list of fairwitnesses\n");
	*b = '\0';
	strcpy(fw,buf);
    }

    /* Read the mode line, if it exists */
    if (fgets(buf,BFSZ,fp) != NULL && mode != NULL)
    {
	if (raw_mode != NULL) raw_mode= strdup(buf);
	b= firstout(buf," \t\n\r");
	if (isdigit(*b) || *b == '-')
	    /* Positive or negative number */
	    *mode= atoi(b);
	    /* Check for 'acl' or 'access' keyword after text mode */
	    b= firstout(firstin(b," \t\n\r")," \t\n\r");
	    if (!strncasecmp(b,"acl",3))
	    	*mode= -1;
	    else if (!strncasecmp(b,"access",6))
	    	*mode= -2;
	else
	{
	    char *err= parse_textmode(b,mode);
	    if (err) die(err);
	}
    }

#ifdef YAPP_COMPAT
    /* Read the email addresses for Yapp maillink conferences - always
     * ignored by Backtalk */

    fgets(buf,BFSZ,fp);
#endif /* YAPP_COMPAT */

    /* Read the conference title */
    if (title != NULL)
    {
	if (fgets(buf,BFSZ,fp) != NULL)
	{
	    if ((b= strchr(buf,'\n')) == NULL)
		die("conference configuration file has overly long "
		    "conference title line\n");
	    *b = '\0';
	    *title= firstout(buf," \t");
	    if (**title == '\0') *title= NULL;
	}
    }

    fclose(fp);
    return;
}


/* WRITECONFIG -- Write out a config file for the currently open conference.  
 * This does not do access control, but it does do some sanity checks.
 * The mode can be given as either a integer or a character string.  The
 * integer version is only used if the string version is NULL.  On success
 * returns NULL.  If errors are found in the data, returns an error message
 * in dynamically allocated memory.
 */

char *writeconfig(char *dir, char *pfname, char *fw, char *textmode,
                  int intmode, char *title)
{
    FILE *fp;
    char buf[BFSZ], *strcontent;
    VSTRING *content;

    /* Sanity check textmode */
    if (textmode != NULL)
    {
	if (index(textmode,'\n') != NULL)
	    return strdup("Conference mode may not contain a newline");
	if (isdigit(*textmode))
	{
	    intmode= atoi(textmode);
	    textmode= NULL;
	}
	else if (*textmode != '-')
	{
	    int mode;
	    char *err= parse_textmode(textmode,&mode);
	    if (err != NULL) return err;
#ifdef PICOSPAN_COMPAT
	    /* For picospan, convert textmodes to numeric values */
	    intmode= mode;
	    textmode= NULL;
#endif
	}
    }

    /* Sanity check pfname */
    if (index(pfname,'\n') != NULL)
	return strdup("Participation file name may not contain a newline");

    /* Sanity check fwlist */
    if (index(fw,'\n') != NULL)
	return strdup("Fairwitness list may not contain a newline");

    /* Sanity check title */
    if (title != NULL && index(title,'\n') != NULL)
	return strdup("Conference title may not contain a newline");

    /* Build an memory image of the new file */
    vs_start(content,256,"!<ps02>\n",7);
    vs_cat_n(content,pfname,firstin(pfname,"\n")-pfname);
    vs_cat(content,"\n0\n");
    vs_cat_n(content,fw,firstin(fw,"\n")-fw);
    *vs_inc(content)= '\n';
    if (textmode == NULL)
    {
    	snprintf(buf,BFSZ,"%d",intmode);
    	vs_cat(content,buf);
    }
    else
	vs_cat(content,textmode);
    *vs_inc(content)= '\n';
    if (title != NULL && *title != '\0')
    {
#ifdef YAPP_COMPAT
	*vs_inc(content)= '\n';
#endif
	vs_cat(content,title);
	*vs_inc(content)= '\n';
    }
    strcontent= vs_return(content);

    /* construct full path name */
    strcpy(buf,dir);
    strcat(buf,CONFIG);

    /* write it out */
    if ((fp= fopen(buf,"w")) == NULL)
	die("could not open %s",buf);
    fputs(strcontent,fp);
    fclose(fp);

    free(strcontent);
}
